// This file is generated by kconfig_compiler_kf6 from kio_ftprc.kcfg.
// All changes you do to this file will be lost.

#include "kio_ftprc.h"

KioFtp::KioFtp( )
  : KConfigSkeleton( QStringLiteral( "kio_ftprc" ) )
{
  setCurrentGroup( QStringLiteral( "<default>" ) );

  KConfigSkeleton::ItemBool  *itemDisablePassiveMode;
  itemDisablePassiveMode = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "DisablePassiveMode" ), mDisablePassiveMode, false );
  addItem( itemDisablePassiveMode, QStringLiteral( "DisablePassiveMode" ) );
  KConfigSkeleton::ItemBool  *itemMarkPartial;
  itemMarkPartial = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "MarkPartial" ), mMarkPartial, true );
  addItem( itemMarkPartial, QStringLiteral( "MarkPartial" ) );
}

KioFtp::~KioFtp()
{
}

