/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kmtp.device.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DEVICEADAPTOR_H
#define DEVICEADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "mtpdevice.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.kmtp.Device
 */
class DeviceAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.kmtp.Device")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.kmtp.Device\">\n"
"    <property access=\"read\" type=\"s\" name=\"udi\"/>\n"
"    <property access=\"read\" type=\"s\" name=\"friendlyName\"/>\n"
"    <method name=\"setFriendlyName\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"newName\"/>\n"
"      <arg direction=\"out\" type=\"i\" name=\"result\"/>\n"
"    </method>\n"
"    <method name=\"listStorages\">\n"
"      <arg direction=\"out\" type=\"ao\" name=\"storages\"/>\n"
"    </method>\n"
"    <method name=\"devicesUpdated\">\n"
"      <arg direction=\"out\" type=\"b\" name=\"devicesUpdated\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    DeviceAdaptor(MTPDevice *parent);
    ~DeviceAdaptor() override;

    inline MTPDevice *parent() const
    { return static_cast<MTPDevice *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QString friendlyName READ friendlyName)
    QString friendlyName() const;

    Q_PROPERTY(QString udi READ udi)
    QString udi() const;

public Q_SLOTS: // METHODS
    bool devicesUpdated();
    QList<QDBusObjectPath> listStorages();
    int setFriendlyName(const QString &newName);
Q_SIGNALS: // SIGNALS
};

#endif
