// This file is generated by kconfig_compiler_kf6 from kioslave.kcfg.
// All changes you do to this file will be lost.

#include "kioslave.h"

ProxySettings::ProxySettings( )
  : KConfigSkeleton( QStringLiteral( "kioslaverc" ) )
{
  setCurrentGroup( QStringLiteral( "<default>" ) );

  KConfigSkeleton::ItemInt  *itemProxyUrlDisplayFlags;
  itemProxyUrlDisplayFlags = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ProxyUrlDisplayFlags" ), mProxyUrlDisplayFlags );
  addItem( itemProxyUrlDisplayFlags, QStringLiteral( "ProxyUrlDisplayFlags" ) );
  KConfigSkeleton::ItemBool  *itemMarkPartial;
  itemMarkPartial = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "MarkPartial" ), mMarkPartial, true );
  addItem( itemMarkPartial, QStringLiteral( "MarkPartial" ) );

  setCurrentGroup( QStringLiteral( "Proxy Settings" ) );

  KConfigSkeleton::ItemString  *itemAuthMode;
  itemAuthMode = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "AuthMode" ), mAuthMode );
  addItem( itemAuthMode, QStringLiteral( "AuthMode" ) );
  KConfigSkeleton::ItemBool  *itemUseReverseProxy;
  itemUseReverseProxy = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ReversedException" ), mUseReverseProxy, false );
  addItem( itemUseReverseProxy, QStringLiteral( "UseReverseProxy" ) );
  KConfigSkeleton::ItemString  *itemFtpProxy;
  itemFtpProxy = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "ftpProxy" ), mFtpProxy );
  addItem( itemFtpProxy, QStringLiteral( "ftpProxy" ) );
  KConfigSkeleton::ItemString  *itemHttpProxy;
  itemHttpProxy = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "httpProxy" ), mHttpProxy );
  addItem( itemHttpProxy, QStringLiteral( "httpProxy" ) );
  KConfigSkeleton::ItemString  *itemHttpsProxy;
  itemHttpsProxy = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "httpsProxy" ), mHttpsProxy );
  addItem( itemHttpsProxy, QStringLiteral( "httpsProxy" ) );
  KConfigSkeleton::ItemString  *itemSocksProxy;
  itemSocksProxy = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "socksProxy" ), mSocksProxy );
  addItem( itemSocksProxy, QStringLiteral( "socksProxy" ) );
  KConfigSkeleton::ItemString  *itemProxyConfigScript;
  itemProxyConfigScript = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Proxy Config Script" ), mProxyConfigScript );
  addItem( itemProxyConfigScript, QStringLiteral( "proxyConfigScript" ) );
  KConfigSkeleton::ItemString  *itemNoProxyFor;
  itemNoProxyFor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "NoProxyFor" ), mNoProxyFor );
  addItem( itemNoProxyFor, QStringLiteral( "NoProxyFor" ) );
  KConfigSkeleton::ItemInt  *itemProxyType;
  itemProxyType = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ProxyType" ), mProxyType );
  addItem( itemProxyType, QStringLiteral( "ProxyType" ) );
}

ProxySettings::~ProxySettings()
{
}

