/****************************************************************************
** Meta object code from reading C++ file 'storageadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "storageadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'storageadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14StorageAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto StorageAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN14StorageAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "StorageAdaptor",
        "D-Bus Interface",
        "org.kde.kmtp.Storage",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.kmtp.Storage\">\n    <property access="
        "\"read\" type=\"s\" name=\"description\"/>\n    <property access=\""
        "read\" type=\"t\" name=\"maxCapacity\"/>\n    <property access=\"r"
        "ead\" type=\"t\" name=\"freeSpaceInBytes\"/>\n    <method name=\"g"
        "etFilesAndFolders\">\n      <arg direction=\"in\" type=\"s\" name="
        "\"path\"/>\n      <arg direction=\"out\" type=\"a(uuustxs)\"/>\n  "
        "    <arg direction=\"out\" type=\"i\" name=\"exists\"/>\n      <an"
        "notation value=\"KMTPFileList\" name=\"org.qtproject.QtDBus.QtType"
        "Name.Out0\"/>\n    </method>\n    <method name=\"getFilesAndFolder"
        "s2\">\n      <arg direction=\"out\" type=\"o\"/>\n      <arg direc"
        "tion=\"in\" type=\"s\" name=\"path\"/>\n    </method>\n    <method"
        " name=\"getFileMetadata\">\n      <arg direction=\"in\" type=\"s\""
        " name=\"path\"/>\n      <arg direction=\"out\" type=\"(uuustxs)\"/"
        ">\n      <annotation value=\"KMTPFile\" name=\"org.qtproject.QtDBu"
        "s.QtTypeName.Out0\"/>\n    </method>\n    <method name=\"getFileTo"
        "Handler\">\n      <arg direction=\"in\" type=\"s\" name=\"path\"/>"
        "\n      <arg direction=\"out\" type=\"i\"/>\n    </method>\n    <m"
        "ethod name=\"getFileToFileDescriptor\">\n      <arg direction=\"in"
        "\" type=\"h\" name=\"descriptor\"/>\n      <arg direction=\"in\" t"
        "ype=\"s\" name=\"sourcePath\"/>\n      <arg direction=\"out\" type"
        "=\"i\" name=\"result\"/>\n    </method>\n    <method name=\"sendFi"
        "leFromFileDescriptor\">\n      <arg direction=\"in\" type=\"h\" na"
        "me=\"descriptor\"/>\n      <arg direction=\"in\" type=\"s\" name=\""
        "destinationPath\"/>\n      <arg direction=\"out\" type=\"i\"/>\n  "
        "  </method>\n    <method name=\"setFileName\">\n      <arg directi"
        "on=\"in\" type=\"s\" name=\"path\"/>\n      <arg direction=\"in\" "
        "type=\"s\" name=\"newName\"/>\n      <arg direction=\"out\" type=\""
        "i\" name=\"result\"/>\n    </method>\n    <method name=\"createFol"
        "der\">\n      <arg direction=\"in\" type=\"s\" name=\"path\"/>\n  "
        "    <arg direction=\"out\" type=\"u\"/>\n    </method>\n    <metho"
        "d name=\"deleteObject\">\n      <arg direction=\"in\" type=\"s\" n"
        "ame=\"path\"/>\n      <arg direction=\"out\" type=\"i\" name=\"res"
        "ult\"/>\n    </method>\n    <signal name=\"dataReady\">\n      <ar"
        "g direction=\"out\" type=\"ay\" name=\"data\"/>\n      <annotation"
        " value=\"QByteArray\" name=\"org.qtproject.QtDBus.QtTypeName.Out0\""
        "/>\n    </signal>\n    <signal name=\"copyProgress\">\n      <arg "
        "type=\"t\" name=\"transferredBytes\"/>\n      <arg type=\"t\" name"
        "=\"totalBytes\"/>\n    </signal>\n    <signal name=\"copyFinished\""
        ">\n      <arg type=\"i\" name=\"result\"/>\n    </signal>\n  </int"
        "erface>\n",
        "copyFinished",
        "",
        "result",
        "copyProgress",
        "transferredBytes",
        "totalBytes",
        "dataReady",
        "data",
        "createFolder",
        "path",
        "deleteObject",
        "getFileMetadata",
        "KMTPFile",
        "getFileToFileDescriptor",
        "QDBusUnixFileDescriptor",
        "descriptor",
        "sourcePath",
        "getFileToHandler",
        "getFilesAndFolders",
        "KMTPFileList",
        "int&",
        "exists",
        "getFilesAndFolders2",
        "QDBusObjectPath",
        "sendFileFromFileDescriptor",
        "destinationPath",
        "setFileName",
        "newName",
        "description",
        "freeSpaceInBytes",
        "maxCapacity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'copyFinished'
        QtMocHelpers::SignalData<void(int)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'copyProgress'
        QtMocHelpers::SignalData<void(qulonglong, qulonglong)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 9 }, { QMetaType::ULongLong, 10 },
        }}),
        // Signal 'dataReady'
        QtMocHelpers::SignalData<void(const QByteArray &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 12 },
        }}),
        // Slot 'createFolder'
        QtMocHelpers::SlotData<uint(const QString &)>(13, 6, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'deleteObject'
        QtMocHelpers::SlotData<int(const QString &)>(15, 6, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'getFileMetadata'
        QtMocHelpers::SlotData<KMTPFile(const QString &)>(16, 6, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'getFileToFileDescriptor'
        QtMocHelpers::SlotData<int(const QDBusUnixFileDescriptor &, const QString &)>(18, 6, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'getFileToHandler'
        QtMocHelpers::SlotData<int(const QString &)>(22, 6, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'getFilesAndFolders'
        QtMocHelpers::SlotData<KMTPFileList(const QString &, int &)>(23, 6, QMC::AccessPublic, 0x80000000 | 24, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'getFilesAndFolders2'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &)>(27, 6, QMC::AccessPublic, 0x80000000 | 28, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'sendFileFromFileDescriptor'
        QtMocHelpers::SlotData<int(const QDBusUnixFileDescriptor &, const QString &)>(29, 6, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 30 },
        }}),
        // Slot 'setFileName'
        QtMocHelpers::SlotData<int(const QString &, const QString &)>(31, 6, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'description'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'freeSpaceInBytes'
        QtMocHelpers::PropertyData<qulonglong>(34, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'maxCapacity'
        QtMocHelpers::PropertyData<qulonglong>(35, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<StorageAdaptor, qt_meta_tag_ZN14StorageAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject StorageAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StorageAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StorageAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14StorageAdaptorE_t>.metaTypes,
    nullptr
} };

void StorageAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<StorageAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->copyFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->copyProgress((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2]))); break;
        case 2: _t->dataReady((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: { uint _r = _t->createFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->deleteObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 5: { KMTPFile _r = _t->getFileMetadata((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KMTPFile*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->getFileToFileDescriptor((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->getFileToHandler((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { KMTPFileList _r = _t->getFilesAndFolders((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[2])));
            if (_a[0]) *reinterpret_cast<KMTPFileList*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusObjectPath _r = _t->getFilesAndFolders2((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->sendFileFromFileDescriptor((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 11: { int _r = _t->setFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (StorageAdaptor::*)(int )>(_a, &StorageAdaptor::copyFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (StorageAdaptor::*)(qulonglong , qulonglong )>(_a, &StorageAdaptor::copyProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (StorageAdaptor::*)(const QByteArray & )>(_a, &StorageAdaptor::dataReady, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 1: *reinterpret_cast<qulonglong*>(_v) = _t->freeSpaceInBytes(); break;
        case 2: *reinterpret_cast<qulonglong*>(_v) = _t->maxCapacity(); break;
        default: break;
        }
    }
}

const QMetaObject *StorageAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StorageAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14StorageAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int StorageAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void StorageAdaptor::copyFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void StorageAdaptor::copyProgress(qulonglong _t1, qulonglong _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void StorageAdaptor::dataReady(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
