/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kmtp.device.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "deviceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DeviceAdaptor
 */

DeviceAdaptor::DeviceAdaptor(MTPDevice *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DeviceAdaptor::~DeviceAdaptor()
{
    // destructor
}

QString DeviceAdaptor::friendlyName() const
{
    // get the value of property friendlyName
    return qvariant_cast< QString >(parent()->property("friendlyName"));
}

QString DeviceAdaptor::udi() const
{
    // get the value of property udi
    return qvariant_cast< QString >(parent()->property("udi"));
}

bool DeviceAdaptor::devicesUpdated()
{
    // handle method call org.kde.kmtp.Device.devicesUpdated
    return parent()->devicesUpdated();
}

QList<QDBusObjectPath> DeviceAdaptor::listStorages()
{
    // handle method call org.kde.kmtp.Device.listStorages
    return parent()->listStorages();
}

int DeviceAdaptor::setFriendlyName(const QString &newName)
{
    // handle method call org.kde.kmtp.Device.setFriendlyName
    return parent()->setFriendlyName(newName);
}


#include "moc_deviceadaptor.cpp"
