// This file is generated by kconfig_compiler_kf6 from jpegcreatorsettings5.kcfg.
// All changes you do to this file will be lost.
#ifndef JPEGCREATORSETTINGS_H
#define JPEGCREATORSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class JpegCreatorSettings : public KConfigSkeleton
{
  public:

    static JpegCreatorSettings *self();
    ~JpegCreatorSettings() override;

    /**
      Set Rotate JPEG automatically
    */
    static
    void setRotate( bool v )
    {
      if (!self()->isRotateImmutable())
        self()->mRotate = v;
    }

    /**
      Get Rotate JPEG automatically
    */
    static
    bool rotate()
    {
      return self()->mRotate;
    }

    /**
      Is Rotate JPEG automatically Immutable
    */
    static
    bool isRotateImmutable()
    {
      return self()->isImmutable( QStringLiteral( "rotate" ) );
    }

  protected:
    JpegCreatorSettings();
    friend class JpegCreatorSettingsHelper;


    // General
    bool mRotate;
};

#endif

