/****************************************************************************
** Meta object code from reading C++ file 'mtpstorage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kio-extras-26.08.0/mtp/kiod_module/mtpstorage.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mtpstorage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MTPStorageE_t {};
} // unnamed namespace

template <> constexpr inline auto MTPStorage::qt_create_metaobjectdata<qt_meta_tag_ZN10MTPStorageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MTPStorage",
        "dataReady",
        "",
        "data",
        "copyProgress",
        "transferredBytes",
        "totalBytes",
        "copyFinished",
        "result",
        "getFilesAndFolders",
        "KMTPFileList",
        "path",
        "int&",
        "getFilesAndFolders2",
        "QDBusObjectPath",
        "getFileMetadata",
        "KMTPFile",
        "getFileToHandler",
        "getFileToFileDescriptor",
        "QDBusUnixFileDescriptor",
        "descriptor",
        "sourcePath",
        "sendFileFromFileDescriptor",
        "destinationPath",
        "setFileName",
        "newName",
        "createFolder",
        "deleteObject",
        "description",
        "maxCapacity",
        "freeSpaceInBytes"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dataReady'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 },
        }}),
        // Signal 'copyProgress'
        QtMocHelpers::SignalData<void(qulonglong, qulonglong)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 5 }, { QMetaType::ULongLong, 6 },
        }}),
        // Signal 'copyFinished'
        QtMocHelpers::SignalData<void(int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Slot 'getFilesAndFolders'
        QtMocHelpers::SlotData<KMTPFileList(const QString &, int &)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 12, 8 },
        }}),
        // Slot 'getFilesAndFolders2'
        QtMocHelpers::SlotData<QDBusObjectPath(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFileMetadata'
        QtMocHelpers::SlotData<KMTPFile(const QString &)>(15, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFileToHandler'
        QtMocHelpers::SlotData<int(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFileToFileDescriptor'
        QtMocHelpers::SlotData<int(const QDBusUnixFileDescriptor &, const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 21 },
        }}),
        // Slot 'sendFileFromFileDescriptor'
        QtMocHelpers::SlotData<int(const QDBusUnixFileDescriptor &, const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 23 },
        }}),
        // Slot 'setFileName'
        QtMocHelpers::SlotData<int(const QString &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 25 },
        }}),
        // Slot 'createFolder'
        QtMocHelpers::SlotData<quint32(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'deleteObject'
        QtMocHelpers::SlotData<int(const QString &)>(27, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'description'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'maxCapacity'
        QtMocHelpers::PropertyData<quint64>(29, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'freeSpaceInBytes'
        QtMocHelpers::PropertyData<quint64>(30, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MTPStorage, qt_meta_tag_ZN10MTPStorageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MTPStorage::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MTPStorageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MTPStorageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MTPStorageE_t>.metaTypes,
    nullptr
} };

void MTPStorage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MTPStorage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dataReady((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->copyProgress((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2]))); break;
        case 2: _t->copyFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: { KMTPFileList _r = _t->getFilesAndFolders((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[2])));
            if (_a[0]) *reinterpret_cast<KMTPFileList*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusObjectPath _r = _t->getFilesAndFolders2((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusObjectPath*>(_a[0]) = std::move(_r); }  break;
        case 5: { KMTPFile _r = _t->getFileMetadata((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<KMTPFile*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->getFileToHandler((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: { int _r = _t->getFileToFileDescriptor((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { int _r = _t->sendFileFromFileDescriptor((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 9: { int _r = _t->setFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: { quint32 _r = _t->createFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<quint32*>(_a[0]) = std::move(_r); }  break;
        case 11: { int _r = _t->deleteObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MTPStorage::*)(const QByteArray & )>(_a, &MTPStorage::dataReady, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MTPStorage::*)(qulonglong , qulonglong )>(_a, &MTPStorage::copyProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MTPStorage::*)(int )>(_a, &MTPStorage::copyFinished, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 1: *reinterpret_cast<quint64*>(_v) = _t->maxCapacity(); break;
        case 2: *reinterpret_cast<quint64*>(_v) = _t->freeSpaceInBytes(); break;
        default: break;
        }
    }
}

const QMetaObject *MTPStorage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MTPStorage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MTPStorageE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int MTPStorage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void MTPStorage::dataReady(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MTPStorage::copyProgress(qulonglong _t1, qulonglong _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MTPStorage::copyFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
