/****************************************************************************
** Meta object code from reading C++ file 'storageinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "storageinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'storageinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgKdeKmtpStorageInterface::qt_create_metaobjectdata<qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgKdeKmtpStorageInterface",
        "copyFinished",
        "",
        "result",
        "copyProgress",
        "transferredBytes",
        "totalBytes",
        "dataReady",
        "data",
        "createFolder",
        "QDBusPendingReply<uint>",
        "path",
        "deleteObject",
        "QDBusPendingReply<int>",
        "getFileMetadata",
        "QDBusPendingReply<KMTPFile>",
        "getFileToFileDescriptor",
        "QDBusUnixFileDescriptor",
        "descriptor",
        "sourcePath",
        "getFileToHandler",
        "getFilesAndFolders",
        "QDBusPendingReply<KMTPFileList,int>",
        "QDBusReply<KMTPFileList>",
        "int&",
        "exists",
        "getFilesAndFolders2",
        "QDBusPendingReply<QDBusObjectPath>",
        "sendFileFromFileDescriptor",
        "destinationPath",
        "setFileName",
        "newName",
        "description",
        "freeSpaceInBytes",
        "maxCapacity"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'copyFinished'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'copyProgress'
        QtMocHelpers::SignalData<void(qulonglong, qulonglong)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 5 }, { QMetaType::ULongLong, 6 },
        }}),
        // Signal 'dataReady'
        QtMocHelpers::SignalData<void(const QByteArray &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 8 },
        }}),
        // Slot 'createFolder'
        QtMocHelpers::SlotData<QDBusPendingReply<uint>(const QString &)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'deleteObject'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFileMetadata'
        QtMocHelpers::SlotData<QDBusPendingReply<KMTPFile>(const QString &)>(14, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFileToFileDescriptor'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QDBusUnixFileDescriptor &, const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 19 },
        }}),
        // Slot 'getFileToHandler'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QString &)>(20, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFilesAndFolders'
        QtMocHelpers::SlotData<QDBusPendingReply<KMTPFileList,int>(const QString &)>(21, 2, QMC::AccessPublic, 0x80000000 | 22, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'getFilesAndFolders'
        QtMocHelpers::SlotData<QDBusReply<KMTPFileList>(const QString &, int &)>(21, 2, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::QString, 11 }, { 0x80000000 | 24, 25 },
        }}),
        // Slot 'getFilesAndFolders2'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusObjectPath>(const QString &)>(26, 2, QMC::AccessPublic, 0x80000000 | 27, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'sendFileFromFileDescriptor'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QDBusUnixFileDescriptor &, const QString &)>(28, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 17, 18 }, { QMetaType::QString, 29 },
        }}),
        // Slot 'setFileName'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QString &, const QString &)>(30, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'description'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'freeSpaceInBytes'
        QtMocHelpers::PropertyData<qulonglong>(33, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'maxCapacity'
        QtMocHelpers::PropertyData<qulonglong>(34, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgKdeKmtpStorageInterface, qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgKdeKmtpStorageInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgKdeKmtpStorageInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgKdeKmtpStorageInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->copyFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->copyProgress((*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2]))); break;
        case 2: _t->dataReady((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: { QDBusPendingReply<uint> _r = _t->createFolder((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<int> _r = _t->deleteObject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<KMTPFile> _r = _t->getFileMetadata((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<KMTPFile>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<int> _r = _t->getFileToFileDescriptor((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<int> _r = _t->getFileToHandler((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<KMTPFileList,int> _r = _t->getFilesAndFolders((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<KMTPFileList,int>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusReply<KMTPFileList> _r = _t->getFilesAndFolders((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int&>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusReply<KMTPFileList>*>(_a[0]) = std::move(_r); }  break;
        case 10: { QDBusPendingReply<QDBusObjectPath> _r = _t->getFilesAndFolders2((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<int> _r = _t->sendFileFromFileDescriptor((*reinterpret_cast<std::add_pointer_t<QDBusUnixFileDescriptor>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<int> _r = _t->setFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKmtpStorageInterface::*)(int )>(_a, &OrgKdeKmtpStorageInterface::copyFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKmtpStorageInterface::*)(qulonglong , qulonglong )>(_a, &OrgKdeKmtpStorageInterface::copyProgress, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OrgKdeKmtpStorageInterface::*)(const QByteArray & )>(_a, &OrgKdeKmtpStorageInterface::dataReady, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        case 1: *reinterpret_cast<qulonglong*>(_v) = _t->freeSpaceInBytes(); break;
        case 2: *reinterpret_cast<qulonglong*>(_v) = _t->maxCapacity(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgKdeKmtpStorageInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgKdeKmtpStorageInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN26OrgKdeKmtpStorageInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgKdeKmtpStorageInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void OrgKdeKmtpStorageInterface::copyFinished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void OrgKdeKmtpStorageInterface::copyProgress(qulonglong _t1, qulonglong _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void OrgKdeKmtpStorageInterface::dataReady(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
