// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "recentlyused-logsettings.h"


Q_LOGGING_CATEGORY(KIO_RECENTLYUSED_LOG, "kf.kio.workers.recentlyused", QtInfoMsg)

