/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kmtp.daemon.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef DAEMONADAPTOR_H
#define DAEMONADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "kmtpd.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.kmtp.Daemon
 */
class DaemonAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.kmtp.Daemon")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.kmtp.Daemon\">\n"
"    <property access=\"read\" type=\"s\" name=\"version\"/>\n"
"    <method name=\"listDevices\">\n"
"      <arg direction=\"out\" type=\"ao\" name=\"devices\"/>\n"
"    </method>\n"
"    <signal name=\"devicesChanged\"/>\n"
"  </interface>\n"
        "")
public:
    DaemonAdaptor(KMTPd *parent);
    ~DaemonAdaptor() override;

    inline KMTPd *parent() const
    { return static_cast<KMTPd *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QString version READ version)
    QString version() const;

public Q_SLOTS: // METHODS
    QList<QDBusObjectPath> listDevices();
Q_SIGNALS: // SIGNALS
    void devicesChanged();
};

#endif
