// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "thumbnail-djvu-logsettings.h"


Q_LOGGING_CATEGORY(KIO_THUMBNAIL_DJVU_LOG, "kf.kio.workers.thumbnail.djvu", QtInfoMsg)

