// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "thumbnail-exr-logsettings.h"


Q_LOGGING_CATEGORY(KIO_THUMBNAIL_EXR_LOG, "kf.kio.workers.thumbnail.exr", QtInfoMsg)

