/****************************************************************************
** Meta object code from reading C++ file 'deviceadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "deviceadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'deviceadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13DeviceAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto DeviceAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN13DeviceAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DeviceAdaptor",
        "D-Bus Interface",
        "org.kde.kmtp.Device",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.kmtp.Device\">\n    <property access=\""
        "read\" type=\"s\" name=\"udi\"/>\n    <property access=\"read\" ty"
        "pe=\"s\" name=\"friendlyName\"/>\n    <method name=\"setFriendlyNa"
        "me\">\n      <arg direction=\"in\" type=\"s\" name=\"newName\"/>\n"
        "      <arg direction=\"out\" type=\"i\" name=\"result\"/>\n    </m"
        "ethod>\n    <method name=\"listStorages\">\n      <arg direction=\""
        "out\" type=\"ao\" name=\"storages\"/>\n    </method>\n    <method "
        "name=\"devicesUpdated\">\n      <arg direction=\"out\" type=\"b\" "
        "name=\"devicesUpdated\"/>\n    </method>\n  </interface>\n",
        "devicesUpdated",
        "",
        "listStorages",
        "QList<QDBusObjectPath>",
        "setFriendlyName",
        "newName",
        "friendlyName",
        "udi"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'devicesUpdated'
        QtMocHelpers::SlotData<bool()>(5, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'listStorages'
        QtMocHelpers::SlotData<QList<QDBusObjectPath>()>(7, 6, QMC::AccessPublic, 0x80000000 | 8),
        // Slot 'setFriendlyName'
        QtMocHelpers::SlotData<int(const QString &)>(9, 6, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'friendlyName'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'udi'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<DeviceAdaptor, qt_meta_tag_ZN13DeviceAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject DeviceAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DeviceAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DeviceAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13DeviceAdaptorE_t>.metaTypes,
    nullptr
} };

void DeviceAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DeviceAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->devicesUpdated();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { QList<QDBusObjectPath> _r = _t->listStorages();
            if (_a[0]) *reinterpret_cast<QList<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->setFriendlyName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->friendlyName(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->udi(); break;
        default: break;
        }
    }
}

const QMetaObject *DeviceAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DeviceAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13DeviceAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int DeviceAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
