/********************************************************************************
** Form generated from reading UI file 'ikwsopts_ui.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IKWSOPTS_UI_H
#define UI_IKWSOPTS_UI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_FilterOptionsUI
{
public:
    QGridLayout *gridLayout;
    QCheckBox *cbEnableShortcuts;
    QLabel *lbQuickHelp;
    QCheckBox *cbUseSelectedShortcutsOnly;
    QLineEdit *searchLineEdit;
    QTreeView *lvSearchProviders;
    QVBoxLayout *verticalLayout;
    QPushButton *pbNew;
    QPushButton *pbChange;
    QPushButton *pbDelete;
    QSpacerItem *verticalSpacer;
    QFormLayout *formLayout;
    QLabel *lbDefaultEngine;
    QComboBox *cmbDefaultEngine;
    QLabel *lbDelimiter;
    QComboBox *cmbDelimiter;

    void setupUi(QWidget *FilterOptionsUI)
    {
        if (FilterOptionsUI->objectName().isEmpty())
            FilterOptionsUI->setObjectName("FilterOptionsUI");
        FilterOptionsUI->resize(475, 404);
        gridLayout = new QGridLayout(FilterOptionsUI);
        gridLayout->setObjectName("gridLayout");
        cbEnableShortcuts = new QCheckBox(FilterOptionsUI);
        cbEnableShortcuts->setObjectName("cbEnableShortcuts");

        gridLayout->addWidget(cbEnableShortcuts, 0, 0, 1, 1);

        lbQuickHelp = new QLabel(FilterOptionsUI);
        lbQuickHelp->setObjectName("lbQuickHelp");
        lbQuickHelp->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lbQuickHelp->sizePolicy().hasHeightForWidth());
        lbQuickHelp->setSizePolicy(sizePolicy);
        lbQuickHelp->setWordWrap(false);
        lbQuickHelp->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::LinksAccessibleByKeyboard);

        gridLayout->addWidget(lbQuickHelp, 0, 1, 1, 1);

        cbUseSelectedShortcutsOnly = new QCheckBox(FilterOptionsUI);
        cbUseSelectedShortcutsOnly->setObjectName("cbUseSelectedShortcutsOnly");
        cbUseSelectedShortcutsOnly->setEnabled(false);

        gridLayout->addWidget(cbUseSelectedShortcutsOnly, 1, 0, 1, 1);

        searchLineEdit = new QLineEdit(FilterOptionsUI);
        searchLineEdit->setObjectName("searchLineEdit");
        searchLineEdit->setProperty("showClearButton", QVariant(true));

        gridLayout->addWidget(searchLineEdit, 2, 0, 1, 1);

        lvSearchProviders = new QTreeView(FilterOptionsUI);
        lvSearchProviders->setObjectName("lvSearchProviders");
        lvSearchProviders->setRootIsDecorated(false);
        lvSearchProviders->setUniformRowHeights(true);
        lvSearchProviders->setItemsExpandable(false);
        lvSearchProviders->setSortingEnabled(true);
        lvSearchProviders->setAllColumnsShowFocus(true);
        lvSearchProviders->setHeaderHidden(false);

        gridLayout->addWidget(lvSearchProviders, 3, 0, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        pbNew = new QPushButton(FilterOptionsUI);
        pbNew->setObjectName("pbNew");

        verticalLayout->addWidget(pbNew);

        pbChange = new QPushButton(FilterOptionsUI);
        pbChange->setObjectName("pbChange");
        pbChange->setEnabled(false);

        verticalLayout->addWidget(pbChange);

        pbDelete = new QPushButton(FilterOptionsUI);
        pbDelete->setObjectName("pbDelete");
        pbDelete->setEnabled(false);

        verticalLayout->addWidget(pbDelete);

        verticalSpacer = new QSpacerItem(20, 158, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        gridLayout->addLayout(verticalLayout, 3, 1, 1, 1);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        lbDefaultEngine = new QLabel(FilterOptionsUI);
        lbDefaultEngine->setObjectName("lbDefaultEngine");
        lbDefaultEngine->setEnabled(true);
        sizePolicy.setHeightForWidth(lbDefaultEngine->sizePolicy().hasHeightForWidth());
        lbDefaultEngine->setSizePolicy(sizePolicy);
        lbDefaultEngine->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, lbDefaultEngine);

        cmbDefaultEngine = new QComboBox(FilterOptionsUI);
        cmbDefaultEngine->setObjectName("cmbDefaultEngine");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cmbDefaultEngine->sizePolicy().hasHeightForWidth());
        cmbDefaultEngine->setSizePolicy(sizePolicy1);
        cmbDefaultEngine->setMinimumSize(QSize(67, 0));
        cmbDefaultEngine->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLengthWithIcon);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, cmbDefaultEngine);

        lbDelimiter = new QLabel(FilterOptionsUI);
        lbDelimiter->setObjectName("lbDelimiter");
        sizePolicy.setHeightForWidth(lbDelimiter->sizePolicy().hasHeightForWidth());
        lbDelimiter->setSizePolicy(sizePolicy);
        lbDelimiter->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, lbDelimiter);

        cmbDelimiter = new QComboBox(FilterOptionsUI);
        cmbDelimiter->addItem(QString());
        cmbDelimiter->addItem(QString());
        cmbDelimiter->setObjectName("cmbDelimiter");
        sizePolicy1.setHeightForWidth(cmbDelimiter->sizePolicy().hasHeightForWidth());
        cmbDelimiter->setSizePolicy(sizePolicy1);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, cmbDelimiter);


        gridLayout->addLayout(formLayout, 4, 0, 1, 1);

#if QT_CONFIG(shortcut)
        lbDefaultEngine->setBuddy(cmbDefaultEngine);
        lbDelimiter->setBuddy(cmbDelimiter);
#endif // QT_CONFIG(shortcut)

        retranslateUi(FilterOptionsUI);
        QObject::connect(cbEnableShortcuts, &QCheckBox::toggled, cbUseSelectedShortcutsOnly, &QCheckBox::setEnabled);
        QObject::connect(cbEnableShortcuts, &QCheckBox::toggled, lvSearchProviders, &QTreeView::setEnabled);
        QObject::connect(cbEnableShortcuts, &QCheckBox::toggled, pbNew, &QPushButton::setEnabled);

        QMetaObject::connectSlotsByName(FilterOptionsUI);
    } // setupUi

    void retranslateUi(QWidget *FilterOptionsUI)
    {
#if QT_CONFIG(tooltip)
        cbEnableShortcuts->setToolTip(tr2i18n("Check to recognise and use the web search keywords in applications that can use them", nullptr));
#endif // QT_CONFIG(tooltip)
        cbEnableShortcuts->setText(tr2i18n("&Enable Web search keywords", nullptr));
        lbQuickHelp->setText(tr2i18n("<a href=\"whatsthis\">How do these work?</a>", nullptr));
#if QT_CONFIG(tooltip)
        cbUseSelectedShortcutsOnly->setToolTip(tr2i18n("<p>Check to only use the preferred web search keywords, as selected in the list below, where necessary.</p><p>If this is checked, then applications that can only show a limited number of search keywords (for example, Konqueror when text on a page is selected) will only show the preferred keywords.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        cbUseSelectedShortcutsOnly->setText(tr2i18n("&Use preferred keywords only", nullptr));
        searchLineEdit->setPlaceholderText(tr2i18n("Search...", nullptr));
#if QT_CONFIG(tooltip)
        pbNew->setToolTip(tr2i18n("Add a new Web search keyword", nullptr));
#endif // QT_CONFIG(tooltip)
        pbNew->setText(tr2i18n("&New\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        pbChange->setToolTip(tr2i18n("Modify the highlighted Web search keyword", nullptr));
#endif // QT_CONFIG(tooltip)
        pbChange->setText(tr2i18n("Ed&it\342\200\246", nullptr));
#if QT_CONFIG(tooltip)
        pbDelete->setToolTip(tr2i18n("Delete the highlighted Web search keyword", nullptr));
#endif // QT_CONFIG(tooltip)
        pbDelete->setText(tr2i18n("De&lete", nullptr));
        lbDefaultEngine->setText(tr2i18n("Default Web &search keyword:", nullptr));
#if QT_CONFIG(tooltip)
        cmbDefaultEngine->setToolTip(tr2i18n("<p>Select a default web search keyword.</p><p>This may be used by applications which automatically search if input cannot be recognised, for example in Konqueror's address bar.  This keyword will be used for the search. To disable this feature select <tt>(None)</tt> from the list.</p>", nullptr));
#endif // QT_CONFIG(tooltip)
        lbDelimiter->setText(tr2i18n("&Keyword delimiter:", nullptr));
        cmbDelimiter->setItemText(0, tr2i18n("Colon", "Colon as keyword delimiter"));
        cmbDelimiter->setItemText(1, tr2i18n("Space", "Space as keyword delimiter"));

#if QT_CONFIG(tooltip)
        cmbDelimiter->setToolTip(tr2i18n("Choose the delimiter to separate the keyword from the phrase or word to be searched.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)FilterOptionsUI;
    } // retranslateUi

};

namespace Ui {
    class FilterOptionsUI: public Ui_FilterOptionsUI {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IKWSOPTS_UI_H

