// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "kio_filenamesearch_debug.h"


Q_LOGGING_CATEGORY(KIO_FILENAMESEARCH, "kf.kio.workers.filenamesearch", QtInfoMsg)

