/********************************************************************************
** Form generated from reading UI file 'kproxydlg.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPROXYDLG_H
#define UI_KPROXYDLG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcontextualhelpbutton.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_ProxyDialogUI
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *verticalSpacer;
    QPushButton *autoDetectButton;
    QRadioButton *noProxyRadioButton;
    QGroupBox *manualProxyGroupBox;
    QFormLayout *formLayout;
    QLabel *manualProxyHttpEditLabel;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *manualProxyHttpEdit;
    QLabel *manualProxyHttpSpinBoxLabel;
    QSpinBox *manualProxyHttpSpinBox;
    QCheckBox *useSameProxyCheckBox;
    QLabel *manualProxyHttpsEditLabel;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *manualProxyHttpsEdit;
    QLabel *manualProxyHttpsSpinBoxLabel;
    QSpinBox *manualProxyHttpsSpinBox;
    QLabel *manualProxyFtpEditLabel;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *manualProxyFtpEdit;
    QLabel *manualProxyFtpSpinBoxLabel;
    QSpinBox *manualProxyFtpSpinBox;
    QLabel *manualProxySocksEditLabel;
    QHBoxLayout *horizontalLayout;
    QLineEdit *manualProxySocksEdit;
    QLabel *manualProxySocksSpinBoxLabel;
    QSpinBox *manualProxySocksSpinBox;
    QLabel *manNoProxyLabel;
    QLineEdit *manualNoProxyEdit;
    QCheckBox *useReverseProxyCheckBox;
    QRadioButton *autoScriptProxyRadioButton;
    QRadioButton *manualProxyRadioButton;
    QGroupBox *systemProxyGroupBox;
    QFormLayout *formLayout_2;
    QLabel *systemProxyHttpLabel;
    QLineEdit *systemProxyHttpEdit;
    QLabel *systemProxyHttpsLabel;
    QLineEdit *systemProxyHttpsEdit;
    QLabel *systemProxyFtpLabel;
    QLineEdit *systemProxyFtpEdit;
    QLabel *systemProxySocksLabel;
    QLineEdit *systemProxySocksEdit;
    QLabel *sysNoProxyLabel;
    QLineEdit *systemNoProxyEdit;
    QCheckBox *showEnvValueCheckBox;
    QHBoxLayout *horizontalLayout_5;
    QRadioButton *autoDiscoverProxyRadioButton;
    KContextualHelpButton *wpadHelpButton;
    QSpacerItem *horizontalSpacer;
    QGroupBox *proxyConfigScriptGroupBox;
    QVBoxLayout *verticalLayout_2;
    KUrlRequester *proxyScriptUrlRequester;
    QRadioButton *systemProxyRadioButton;

    void setupUi(QWidget *ProxyDialogUI)
    {
        if (ProxyDialogUI->objectName().isEmpty())
            ProxyDialogUI->setObjectName("ProxyDialogUI");
        ProxyDialogUI->resize(513, 728);
        gridLayout = new QGridLayout(ProxyDialogUI);
        gridLayout->setObjectName("gridLayout");
        verticalSpacer = new QSpacerItem(20, 42, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout->addItem(verticalSpacer, 8, 0, 1, 4);

        autoDetectButton = new QPushButton(ProxyDialogUI);
        autoDetectButton->setObjectName("autoDetectButton");

        gridLayout->addWidget(autoDetectButton, 4, 3, 1, 1);

        noProxyRadioButton = new QRadioButton(ProxyDialogUI);
        noProxyRadioButton->setObjectName("noProxyRadioButton");
        noProxyRadioButton->setChecked(true);

        gridLayout->addWidget(noProxyRadioButton, 0, 0, 1, 1);

        manualProxyGroupBox = new QGroupBox(ProxyDialogUI);
        manualProxyGroupBox->setObjectName("manualProxyGroupBox");
        manualProxyGroupBox->setFlat(true);
        formLayout = new QFormLayout(manualProxyGroupBox);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldGrowthPolicy::ExpandingFieldsGrow);
        formLayout->setContentsMargins(20, 0, 0, 0);
        manualProxyHttpEditLabel = new QLabel(manualProxyGroupBox);
        manualProxyHttpEditLabel->setObjectName("manualProxyHttpEditLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(manualProxyHttpEditLabel->sizePolicy().hasHeightForWidth());
        manualProxyHttpEditLabel->setSizePolicy(sizePolicy);
        manualProxyHttpEditLabel->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, manualProxyHttpEditLabel);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        manualProxyHttpEdit = new QLineEdit(manualProxyGroupBox);
        manualProxyHttpEdit->setObjectName("manualProxyHttpEdit");

        horizontalLayout_4->addWidget(manualProxyHttpEdit);

        manualProxyHttpSpinBoxLabel = new QLabel(manualProxyGroupBox);
        manualProxyHttpSpinBoxLabel->setObjectName("manualProxyHttpSpinBoxLabel");

        horizontalLayout_4->addWidget(manualProxyHttpSpinBoxLabel);

        manualProxyHttpSpinBox = new QSpinBox(manualProxyGroupBox);
        manualProxyHttpSpinBox->setObjectName("manualProxyHttpSpinBox");
        manualProxyHttpSpinBox->setMaximumSize(QSize(32767, 32767));
        manualProxyHttpSpinBox->setMaximum(65536);

        horizontalLayout_4->addWidget(manualProxyHttpSpinBox);


        formLayout->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_4);

        useSameProxyCheckBox = new QCheckBox(manualProxyGroupBox);
        useSameProxyCheckBox->setObjectName("useSameProxyCheckBox");
        useSameProxyCheckBox->setEnabled(false);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, useSameProxyCheckBox);

        manualProxyHttpsEditLabel = new QLabel(manualProxyGroupBox);
        manualProxyHttpsEditLabel->setObjectName("manualProxyHttpsEditLabel");
        sizePolicy.setHeightForWidth(manualProxyHttpsEditLabel->sizePolicy().hasHeightForWidth());
        manualProxyHttpsEditLabel->setSizePolicy(sizePolicy);
        manualProxyHttpsEditLabel->setWordWrap(false);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, manualProxyHttpsEditLabel);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        manualProxyHttpsEdit = new QLineEdit(manualProxyGroupBox);
        manualProxyHttpsEdit->setObjectName("manualProxyHttpsEdit");

        horizontalLayout_3->addWidget(manualProxyHttpsEdit);

        manualProxyHttpsSpinBoxLabel = new QLabel(manualProxyGroupBox);
        manualProxyHttpsSpinBoxLabel->setObjectName("manualProxyHttpsSpinBoxLabel");

        horizontalLayout_3->addWidget(manualProxyHttpsSpinBoxLabel);

        manualProxyHttpsSpinBox = new QSpinBox(manualProxyGroupBox);
        manualProxyHttpsSpinBox->setObjectName("manualProxyHttpsSpinBox");
        manualProxyHttpsSpinBox->setMaximum(65536);

        horizontalLayout_3->addWidget(manualProxyHttpsSpinBox);


        formLayout->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout_3);

        manualProxyFtpEditLabel = new QLabel(manualProxyGroupBox);
        manualProxyFtpEditLabel->setObjectName("manualProxyFtpEditLabel");
        sizePolicy.setHeightForWidth(manualProxyFtpEditLabel->sizePolicy().hasHeightForWidth());
        manualProxyFtpEditLabel->setSizePolicy(sizePolicy);
        manualProxyFtpEditLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        manualProxyFtpEditLabel->setWordWrap(false);

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, manualProxyFtpEditLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        manualProxyFtpEdit = new QLineEdit(manualProxyGroupBox);
        manualProxyFtpEdit->setObjectName("manualProxyFtpEdit");

        horizontalLayout_2->addWidget(manualProxyFtpEdit);

        manualProxyFtpSpinBoxLabel = new QLabel(manualProxyGroupBox);
        manualProxyFtpSpinBoxLabel->setObjectName("manualProxyFtpSpinBoxLabel");

        horizontalLayout_2->addWidget(manualProxyFtpSpinBoxLabel);

        manualProxyFtpSpinBox = new QSpinBox(manualProxyGroupBox);
        manualProxyFtpSpinBox->setObjectName("manualProxyFtpSpinBox");
        manualProxyFtpSpinBox->setMaximum(65536);

        horizontalLayout_2->addWidget(manualProxyFtpSpinBox);


        formLayout->setLayout(3, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);

        manualProxySocksEditLabel = new QLabel(manualProxyGroupBox);
        manualProxySocksEditLabel->setObjectName("manualProxySocksEditLabel");
        sizePolicy.setHeightForWidth(manualProxySocksEditLabel->sizePolicy().hasHeightForWidth());
        manualProxySocksEditLabel->setSizePolicy(sizePolicy);
        manualProxySocksEditLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, manualProxySocksEditLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        manualProxySocksEdit = new QLineEdit(manualProxyGroupBox);
        manualProxySocksEdit->setObjectName("manualProxySocksEdit");

        horizontalLayout->addWidget(manualProxySocksEdit);

        manualProxySocksSpinBoxLabel = new QLabel(manualProxyGroupBox);
        manualProxySocksSpinBoxLabel->setObjectName("manualProxySocksSpinBoxLabel");

        horizontalLayout->addWidget(manualProxySocksSpinBoxLabel);

        manualProxySocksSpinBox = new QSpinBox(manualProxyGroupBox);
        manualProxySocksSpinBox->setObjectName("manualProxySocksSpinBox");
        manualProxySocksSpinBox->setMaximum(65536);

        horizontalLayout->addWidget(manualProxySocksSpinBox);


        formLayout->setLayout(4, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        manNoProxyLabel = new QLabel(manualProxyGroupBox);
        manNoProxyLabel->setObjectName("manNoProxyLabel");
        manNoProxyLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        manNoProxyLabel->setWordWrap(false);

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, manNoProxyLabel);

        manualNoProxyEdit = new QLineEdit(manualProxyGroupBox);
        manualNoProxyEdit->setObjectName("manualNoProxyEdit");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(manualNoProxyEdit->sizePolicy().hasHeightForWidth());
        manualNoProxyEdit->setSizePolicy(sizePolicy1);

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, manualNoProxyEdit);

        useReverseProxyCheckBox = new QCheckBox(manualProxyGroupBox);
        useReverseProxyCheckBox->setObjectName("useReverseProxyCheckBox");
        useReverseProxyCheckBox->setEnabled(false);

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, useReverseProxyCheckBox);


        gridLayout->addWidget(manualProxyGroupBox, 7, 0, 1, 4);

        autoScriptProxyRadioButton = new QRadioButton(ProxyDialogUI);
        autoScriptProxyRadioButton->setObjectName("autoScriptProxyRadioButton");
        autoScriptProxyRadioButton->setEnabled(true);

        gridLayout->addWidget(autoScriptProxyRadioButton, 2, 0, 1, 2);

        manualProxyRadioButton = new QRadioButton(ProxyDialogUI);
        manualProxyRadioButton->setObjectName("manualProxyRadioButton");
        manualProxyRadioButton->setEnabled(true);
        QSizePolicy sizePolicy2(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(manualProxyRadioButton->sizePolicy().hasHeightForWidth());
        manualProxyRadioButton->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(manualProxyRadioButton, 6, 0, 1, 2);

        systemProxyGroupBox = new QGroupBox(ProxyDialogUI);
        systemProxyGroupBox->setObjectName("systemProxyGroupBox");
        systemProxyGroupBox->setFlat(true);
        formLayout_2 = new QFormLayout(systemProxyGroupBox);
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setContentsMargins(20, 0, 0, 0);
        systemProxyHttpLabel = new QLabel(systemProxyGroupBox);
        systemProxyHttpLabel->setObjectName("systemProxyHttpLabel");
        systemProxyHttpLabel->setWordWrap(false);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, systemProxyHttpLabel);

        systemProxyHttpEdit = new QLineEdit(systemProxyGroupBox);
        systemProxyHttpEdit->setObjectName("systemProxyHttpEdit");
        sizePolicy1.setHeightForWidth(systemProxyHttpEdit->sizePolicy().hasHeightForWidth());
        systemProxyHttpEdit->setSizePolicy(sizePolicy1);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, systemProxyHttpEdit);

        systemProxyHttpsLabel = new QLabel(systemProxyGroupBox);
        systemProxyHttpsLabel->setObjectName("systemProxyHttpsLabel");
        systemProxyHttpsLabel->setWordWrap(false);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, systemProxyHttpsLabel);

        systemProxyHttpsEdit = new QLineEdit(systemProxyGroupBox);
        systemProxyHttpsEdit->setObjectName("systemProxyHttpsEdit");
        sizePolicy1.setHeightForWidth(systemProxyHttpsEdit->sizePolicy().hasHeightForWidth());
        systemProxyHttpsEdit->setSizePolicy(sizePolicy1);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, systemProxyHttpsEdit);

        systemProxyFtpLabel = new QLabel(systemProxyGroupBox);
        systemProxyFtpLabel->setObjectName("systemProxyFtpLabel");
        systemProxyFtpLabel->setWordWrap(false);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, systemProxyFtpLabel);

        systemProxyFtpEdit = new QLineEdit(systemProxyGroupBox);
        systemProxyFtpEdit->setObjectName("systemProxyFtpEdit");
        sizePolicy1.setHeightForWidth(systemProxyFtpEdit->sizePolicy().hasHeightForWidth());
        systemProxyFtpEdit->setSizePolicy(sizePolicy1);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, systemProxyFtpEdit);

        systemProxySocksLabel = new QLabel(systemProxyGroupBox);
        systemProxySocksLabel->setObjectName("systemProxySocksLabel");
        systemProxySocksLabel->setWordWrap(false);

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, systemProxySocksLabel);

        systemProxySocksEdit = new QLineEdit(systemProxyGroupBox);
        systemProxySocksEdit->setObjectName("systemProxySocksEdit");
        sizePolicy1.setHeightForWidth(systemProxySocksEdit->sizePolicy().hasHeightForWidth());
        systemProxySocksEdit->setSizePolicy(sizePolicy1);

        formLayout_2->setWidget(3, QFormLayout::ItemRole::FieldRole, systemProxySocksEdit);

        sysNoProxyLabel = new QLabel(systemProxyGroupBox);
        sysNoProxyLabel->setObjectName("sysNoProxyLabel");
        sysNoProxyLabel->setWordWrap(false);

        formLayout_2->setWidget(4, QFormLayout::ItemRole::LabelRole, sysNoProxyLabel);

        systemNoProxyEdit = new QLineEdit(systemProxyGroupBox);
        systemNoProxyEdit->setObjectName("systemNoProxyEdit");
        sizePolicy1.setHeightForWidth(systemNoProxyEdit->sizePolicy().hasHeightForWidth());
        systemNoProxyEdit->setSizePolicy(sizePolicy1);

        formLayout_2->setWidget(4, QFormLayout::ItemRole::FieldRole, systemNoProxyEdit);

        showEnvValueCheckBox = new QCheckBox(systemProxyGroupBox);
        showEnvValueCheckBox->setObjectName("showEnvValueCheckBox");

        formLayout_2->setWidget(5, QFormLayout::ItemRole::FieldRole, showEnvValueCheckBox);


        gridLayout->addWidget(systemProxyGroupBox, 5, 0, 1, 4);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        autoDiscoverProxyRadioButton = new QRadioButton(ProxyDialogUI);
        autoDiscoverProxyRadioButton->setObjectName("autoDiscoverProxyRadioButton");
        autoDiscoverProxyRadioButton->setEnabled(true);

        horizontalLayout_5->addWidget(autoDiscoverProxyRadioButton);

        wpadHelpButton = new KContextualHelpButton(ProxyDialogUI);
        wpadHelpButton->setObjectName("wpadHelpButton");
        wpadHelpButton->setLayoutDirection(Qt::LayoutDirection::LeftToRight);

        horizontalLayout_5->addWidget(wpadHelpButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout_5, 1, 0, 1, 4);

        proxyConfigScriptGroupBox = new QGroupBox(ProxyDialogUI);
        proxyConfigScriptGroupBox->setObjectName("proxyConfigScriptGroupBox");
        proxyConfigScriptGroupBox->setFlat(true);
        verticalLayout_2 = new QVBoxLayout(proxyConfigScriptGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(20, 0, 0, 0);
        proxyScriptUrlRequester = new KUrlRequester(proxyConfigScriptGroupBox);
        proxyScriptUrlRequester->setObjectName("proxyScriptUrlRequester");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(proxyScriptUrlRequester->sizePolicy().hasHeightForWidth());
        proxyScriptUrlRequester->setSizePolicy(sizePolicy3);

        verticalLayout_2->addWidget(proxyScriptUrlRequester);


        gridLayout->addWidget(proxyConfigScriptGroupBox, 3, 0, 1, 4);

        systemProxyRadioButton = new QRadioButton(ProxyDialogUI);
        systemProxyRadioButton->setObjectName("systemProxyRadioButton");
        systemProxyRadioButton->setEnabled(true);
        sizePolicy2.setHeightForWidth(systemProxyRadioButton->sizePolicy().hasHeightForWidth());
        systemProxyRadioButton->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(systemProxyRadioButton, 4, 0, 1, 1);

#if QT_CONFIG(shortcut)
        manualProxyHttpEditLabel->setBuddy(manualProxyHttpEdit);
        manualProxyHttpSpinBoxLabel->setBuddy(manualProxyHttpSpinBox);
        manualProxyHttpsEditLabel->setBuddy(manualProxyHttpsEdit);
        manualProxyHttpsSpinBoxLabel->setBuddy(manualProxyHttpsSpinBox);
        manualProxyFtpEditLabel->setBuddy(manualProxyFtpEdit);
        manualProxyFtpSpinBoxLabel->setBuddy(manualProxyFtpSpinBox);
        manualProxySocksEditLabel->setBuddy(manualProxySocksEdit);
        manualProxySocksSpinBoxLabel->setBuddy(manualProxySocksSpinBox);
        manNoProxyLabel->setBuddy(manualNoProxyEdit);
        systemProxyHttpLabel->setBuddy(systemProxyHttpEdit);
        systemProxyHttpsLabel->setBuddy(systemProxyHttpsEdit);
        systemProxyFtpLabel->setBuddy(systemProxyFtpEdit);
        systemProxySocksLabel->setBuddy(systemProxySocksEdit);
        sysNoProxyLabel->setBuddy(systemNoProxyEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(noProxyRadioButton, autoDiscoverProxyRadioButton);
        QWidget::setTabOrder(autoDiscoverProxyRadioButton, autoScriptProxyRadioButton);
        QWidget::setTabOrder(autoScriptProxyRadioButton, proxyScriptUrlRequester);
        QWidget::setTabOrder(proxyScriptUrlRequester, systemProxyRadioButton);
        QWidget::setTabOrder(systemProxyRadioButton, autoDetectButton);
        QWidget::setTabOrder(autoDetectButton, systemProxyHttpEdit);
        QWidget::setTabOrder(systemProxyHttpEdit, systemProxyHttpsEdit);
        QWidget::setTabOrder(systemProxyHttpsEdit, systemProxyFtpEdit);
        QWidget::setTabOrder(systemProxyFtpEdit, systemProxySocksEdit);
        QWidget::setTabOrder(systemProxySocksEdit, systemNoProxyEdit);
        QWidget::setTabOrder(systemNoProxyEdit, showEnvValueCheckBox);
        QWidget::setTabOrder(showEnvValueCheckBox, manualProxyRadioButton);
        QWidget::setTabOrder(manualProxyRadioButton, manualProxyHttpEdit);
        QWidget::setTabOrder(manualProxyHttpEdit, manualProxyHttpSpinBox);
        QWidget::setTabOrder(manualProxyHttpSpinBox, useSameProxyCheckBox);
        QWidget::setTabOrder(useSameProxyCheckBox, manualProxyHttpsEdit);
        QWidget::setTabOrder(manualProxyHttpsEdit, manualProxyHttpsSpinBox);
        QWidget::setTabOrder(manualProxyHttpsSpinBox, manualProxyFtpEdit);
        QWidget::setTabOrder(manualProxyFtpEdit, manualProxyFtpSpinBox);
        QWidget::setTabOrder(manualProxyFtpSpinBox, manualProxySocksEdit);
        QWidget::setTabOrder(manualProxySocksEdit, manualProxySocksSpinBox);
        QWidget::setTabOrder(manualProxySocksSpinBox, manualNoProxyEdit);
        QWidget::setTabOrder(manualNoProxyEdit, useReverseProxyCheckBox);

        retranslateUi(ProxyDialogUI);
        QObject::connect(systemProxyRadioButton, &QRadioButton::toggled, systemProxyGroupBox, &QGroupBox::setVisible);
        QObject::connect(manualProxyRadioButton, &QRadioButton::toggled, manualProxyGroupBox, &QGroupBox::setVisible);
        QObject::connect(useSameProxyCheckBox, &QCheckBox::toggled, manualProxyHttpsEdit, &QLineEdit::setDisabled);
        QObject::connect(useSameProxyCheckBox, &QCheckBox::toggled, manualProxyFtpEdit, &QLineEdit::setDisabled);
        QObject::connect(useSameProxyCheckBox, &QCheckBox::toggled, manualProxySocksEdit, &QLineEdit::setDisabled);
        QObject::connect(showEnvValueCheckBox, &QCheckBox::toggled, systemNoProxyEdit, &QLineEdit::setDisabled);
        QObject::connect(showEnvValueCheckBox, &QCheckBox::toggled, systemProxySocksEdit, &QLineEdit::setDisabled);
        QObject::connect(showEnvValueCheckBox, &QCheckBox::toggled, systemProxyFtpEdit, &QLineEdit::setDisabled);
        QObject::connect(showEnvValueCheckBox, &QCheckBox::toggled, systemProxyHttpsEdit, &QLineEdit::setDisabled);
        QObject::connect(showEnvValueCheckBox, &QCheckBox::toggled, systemProxyHttpEdit, &QLineEdit::setDisabled);
        QObject::connect(autoScriptProxyRadioButton, &QRadioButton::toggled, proxyConfigScriptGroupBox, &QGroupBox::setVisible);
        QObject::connect(systemProxyRadioButton, &QRadioButton::toggled, autoDetectButton, &QPushButton::setVisible);
        QObject::connect(useSameProxyCheckBox, &QCheckBox::toggled, manualProxyHttpsSpinBox, &QSpinBox::setDisabled);
        QObject::connect(useSameProxyCheckBox, &QCheckBox::toggled, manualProxyFtpSpinBox, &QSpinBox::setDisabled);
        QObject::connect(useSameProxyCheckBox, &QCheckBox::toggled, manualProxySocksSpinBox, &QSpinBox::setDisabled);

        QMetaObject::connectSlotsByName(ProxyDialogUI);
    } // setupUi

    void retranslateUi(QWidget *ProxyDialogUI)
    {
#if QT_CONFIG(whatsthis)
        ProxyDialogUI->setWhatsThis(tr2i18n("<qt>\n"
"Setup proxy configuration.\n"
"<p>\n"
"A proxy server is an intermediate machine that sits between your computer and the Internet and provides services such as web page caching and filtering. Caching proxy servers give you faster access to web sites you have already visited by locally storing or caching those pages; filtering proxy servers usually provide the ability to block out requests for ads, spam, or anything else you want to block.\n"
"<p>\n"
"If you are uncertain whether or not you need to use a proxy server to connect to the Internet, consult your Internet service provider's setup guide or your system administrator.\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        autoDetectButton->setWhatsThis(tr2i18n("<qt>Attempt automatic discovery of the environment variables used for setting system wide proxy information.<p> This feature works by searching for commonly used variable names such as HTTP_PROXY, FTP_PROXY and NO_PROXY.</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        autoDetectButton->setText(tr2i18n("Auto D&etect", nullptr));
#if QT_CONFIG(whatsthis)
        noProxyRadioButton->setWhatsThis(tr2i18n("Connect to the Internet directly.", nullptr));
#endif // QT_CONFIG(whatsthis)
        noProxyRadioButton->setText(tr2i18n("&No proxy", nullptr));
        manualProxyGroupBox->setTitle(QString());
        manualProxyHttpEditLabel->setText(tr2i18n("HTTP proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyHttpEdit->setWhatsThis(tr2i18n("Enter the address of the HTTP proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxyHttpSpinBoxLabel->setText(tr2i18n("Port:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyHttpSpinBox->setWhatsThis(tr2i18n("Enter the port number of the HTTP proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        useSameProxyCheckBox->setText(tr2i18n("Use this proxy server for a&ll protocols", nullptr));
        manualProxyHttpsEditLabel->setText(tr2i18n("SSL proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyHttpsEdit->setWhatsThis(tr2i18n("Enter the address of the HTTPS proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxyHttpsSpinBoxLabel->setText(tr2i18n("Port:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyHttpsSpinBox->setWhatsThis(tr2i18n("Enter the port number of the HTTPS proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxyFtpEditLabel->setText(tr2i18n("FTP proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyFtpEdit->setWhatsThis(tr2i18n("Enter the address of the FTP proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxyFtpSpinBoxLabel->setText(tr2i18n("Port:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyFtpSpinBox->setWhatsThis(tr2i18n("Enter the port number of the FTP proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxySocksEditLabel->setText(tr2i18n("SOCKS proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxySocksEdit->setWhatsThis(tr2i18n("Enter the address of the SOCKS proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxySocksSpinBoxLabel->setText(tr2i18n("Port:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxySocksSpinBox->setWhatsThis(tr2i18n("Enter the port number of the SOCKS proxy server.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        manNoProxyLabel->setWhatsThis(tr2i18n("<qt>\n"
"Enter the environment variable, e.g. <b>NO_PROXY</b>, used to store the addresses of sites for which the proxy server should not be used.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        manNoProxyLabel->setText(tr2i18n("Exceptions:", nullptr));
#if QT_CONFIG(whatsthis)
        manualNoProxyEdit->setWhatsThis(tr2i18n("<qt>\n"
"<p>Enter a comma separated list of hostnames or ip addresses that should be excluded from using the above proxy settings.</p>\n"
"<p>If you want to exclude all hosts for a given domain, then simply enter the domain name preceded by a dot. For example, to exclude all hostnames for <i>kde.org</i>, enter <i>.kde.org</i>. Wildcard characters such as '*' or '?' are not supported and will have no effect.</p>\n"
"<p>Additionally, you can also enter IP addresses, e.g. 127.0.0.1 and IP addresses with a subnet, e.g. 192.168.0.1/24.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        useReverseProxyCheckBox->setWhatsThis(tr2i18n("<qt>\n"
"Check this box if you want the above proxy settings to apply only to the addresses listed in the <i>Exceptions</i> list.</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        useReverseProxyCheckBox->setText(tr2i18n("Use proxy settings only for addresses in the exceptions list", nullptr));
#if QT_CONFIG(whatsthis)
        autoScriptProxyRadioButton->setWhatsThis(tr2i18n("Use the specified proxy script to configure the proxy settings.", nullptr));
#endif // QT_CONFIG(whatsthis)
        autoScriptProxyRadioButton->setText(tr2i18n("&Use proxy auto configuration URL:", nullptr));
#if QT_CONFIG(whatsthis)
        manualProxyRadioButton->setWhatsThis(tr2i18n("Manually enter proxy server configuration information.", nullptr));
#endif // QT_CONFIG(whatsthis)
        manualProxyRadioButton->setText(tr2i18n("Use manually specified proxy configuration:", nullptr));
#if QT_CONFIG(whatsthis)
        systemProxyHttpLabel->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>HTTP_PROXY</b>, used to store the address of the HTTP proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        systemProxyHttpLabel->setText(tr2i18n("H&TTP proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        systemProxyHttpEdit->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>HTTP_PROXY</b>, used to store the address of the HTTP proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        systemProxyHttpsLabel->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>HTTPS_PROXY</b>, used to store the address of the HTTPS proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        systemProxyHttpsLabel->setText(tr2i18n("SS&L proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        systemProxyHttpsEdit->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>HTTPS_PROXY</b>, used to store the address of the HTTPS proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        systemProxyFtpLabel->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>FTP_PROXY</b>, used to store the address of the FTP proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        systemProxyFtpLabel->setText(tr2i18n("FTP proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        systemProxyFtpEdit->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>FTP_PROXY</b>, used to store the address of the FTP proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        systemProxySocksLabel->setWhatsThis(tr2i18n("<qt>\n"
"Enter the name of the environment variable, e.g. <b>SOCKS_PROXY</b>, used to store the address of the SOCKS proxy server.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        systemProxySocksLabel->setText(tr2i18n("SOC&KS proxy:", nullptr));
#if QT_CONFIG(whatsthis)
        systemProxySocksEdit->setWhatsThis(tr2i18n("<qt>Enter the name of the environment variable, e.g. <b>SOCKS_PROXY</b>, used to store the address of the SOCKS proxy server.<p>Alternatively, you can click on the <b>&quot;Auto Detect&quot;</b> button to attempt an automatic discovery of this variable.</p></qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        sysNoProxyLabel->setWhatsThis(tr2i18n("<qt>\n"
"Enter the environment variable, e.g. <b>NO_PROXY</b>, used to store the addresses of sites for which the proxy server should not be used.<p>\n"
"Alternatively, you can click on the <b>\"Auto Detect\"</b> button to attempt an automatic discovery of this variable.\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        sysNoProxyLabel->setText(tr2i18n("E&xceptions:", nullptr));
#if QT_CONFIG(whatsthis)
        systemNoProxyEdit->setWhatsThis(tr2i18n("<qt>Enter the environment variable, e.g. <b>NO_PROXY</b>, used to store the addresses of sites for which the above proxy settings should not be used.<p>Alternatively, you can click on the <b>&quot;Auto Detect&quot;</b> button to attempt an automatic discovery of this variable.</p></qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        showEnvValueCheckBox->setText(tr2i18n("Show the &value of the environment variables", nullptr));
#if QT_CONFIG(whatsthis)
        autoDiscoverProxyRadioButton->setWhatsThis(tr2i18n("<qt>\n"
"Automatically detect and configure the proxy settings.<p>\n"
"Automatic detection is performed using the <b>Web Proxy Auto-Discovery Protocol (WPAD)</b>.<p>\n"
"<b>NOTE:</b> This option might not work properly or not work at all in some UNIX/Linux distributions.\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        autoDiscoverProxyRadioButton->setText(tr2i18n("&Detect proxy configuration automatically", nullptr));
        wpadHelpButton->setContextualHelpText(tr2i18n("Automatic Configuration, also known as Web Proxy Auto-Discovery (WPAD), is almost never the correct configuration. Enabling it when it is not required may slow down network access and make your network less secure. Your network administrator will know whether enabling it is required.", nullptr));
#if QT_CONFIG(whatsthis)
        proxyScriptUrlRequester->setWhatsThis(tr2i18n("Enter the address for the proxy configuration script.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        systemProxyRadioButton->setWhatsThis(tr2i18n("<qt><p>Use proxy settings defined on the system.</p>\n"
"<p>Some platforms offer system wide proxy configuration information and selecting this option allows you to use those settings.</p>\n"
"<p>On Mac platforms</p>\n"
"<p>On Windows platforms</p>\n"
"<p>On Unix and Linux platforms, such system proxy settings are usually defined through environment variables. The following environment variables are detected and used when present: <b>HTTP_PROXY</b>, <b>HTTPS_PROXY</b>, <b>FTP_PROXY</b>, <b>NO_PROXY</b>.</p>\n"
"</qt>", nullptr));
#endif // QT_CONFIG(whatsthis)
        systemProxyRadioButton->setText(tr2i18n("Use s&ystem proxy configuration:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ProxyDialogUI: public Ui_ProxyDialogUI {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPROXYDLG_H

