// This file is generated by kconfig_compiler_kf6 from kio_ftprc.kcfg.
// All changes you do to this file will be lost.
#ifndef KIOFTP_H
#define KIOFTP_H

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class KioFtp : public KConfigSkeleton
{
  public:

    KioFtp( );
    ~KioFtp() override;

    /**
      Set Disable Passive FTP
    */
    void setDisablePassiveMode( bool v )
    {
      if (!isDisablePassiveModeImmutable())
        mDisablePassiveMode = v;
    }

    /**
      Get Disable Passive FTP
    */
    bool disablePassiveMode() const
    {
      return mDisablePassiveMode;
    }

    /**
      Is Disable Passive FTP Immutable
    */
    bool isDisablePassiveModeImmutable() const
    {
      return isImmutable( QStringLiteral( "DisablePassiveMode" ) );
    }

    /**
      Set Mark partially uploaded files
    */
    void setMarkPartial( bool v )
    {
      if (!isMarkPartialImmutable())
        mMarkPartial = v;
    }

    /**
      Get Mark partially uploaded files
    */
    bool markPartial() const
    {
      return mMarkPartial;
    }

    /**
      Is Mark partially uploaded files Immutable
    */
    bool isMarkPartialImmutable() const
    {
      return isImmutable( QStringLiteral( "MarkPartial" ) );
    }

  protected:

    // <default>
    bool mDisablePassiveMode;
    bool mMarkPartial;
};

#endif

